@@ -1,6 +1,4 @@
-# file openpyxl/writer/excel.py
-
-# Copyright (c) 2010-2011 openpyxl
+# Copyright (c) 2010-2014 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,28 +25,42 @@
 
 # Python stdlib imports
 from zipfile import ZipFile, ZIP_DEFLATED
-try:
-    # Python 2
-    from StringIO import StringIO
-    BytesIO = StringIO
-except ImportError:
-    # Python 3
-    from io import BytesIO, StringIO
+
+# compatibility imports
+from openpyxl.shared.compat import BytesIO, StringIO
 
 # package imports
-from openpyxl.shared.ooxml import ARC_SHARED_STRINGS, ARC_CONTENT_TYPES, \
-        ARC_ROOT_RELS, ARC_WORKBOOK_RELS, ARC_APP, ARC_CORE, ARC_THEME, \
-        ARC_STYLE, ARC_WORKBOOK, \
-        PACKAGE_WORKSHEETS, PACKAGE_DRAWINGS, PACKAGE_CHARTS, PACKAGE_IMAGES
+from openpyxl.shared.ooxml import (
+    ARC_SHARED_STRINGS,
+    ARC_CONTENT_TYPES,
+    ARC_ROOT_RELS,
+    ARC_WORKBOOK_RELS,
+    ARC_APP, ARC_CORE,
+    ARC_THEME,
+    ARC_STYLE,
+    ARC_WORKBOOK,
+    ARC_VBA,
+    PACKAGE_WORKSHEETS,
+    PACKAGE_DRAWINGS,
+    PACKAGE_CHARTS,
+    PACKAGE_IMAGES,
+    PACKAGE_XL
+    )
 from openpyxl.writer.strings import create_string_table, write_string_table
-from openpyxl.writer.workbook import write_content_types, write_root_rels, \
-        write_workbook_rels, write_properties_app, write_properties_core, \
-        write_workbook
+from openpyxl.writer.workbook import (
+    write_content_types,
+    write_root_rels,
+    write_workbook_rels,
+    write_properties_app,
+    write_properties_core,
+    write_workbook
+    )
 from openpyxl.writer.theme import write_theme
 from openpyxl.writer.styles import StyleWriter
 from openpyxl.writer.drawings import DrawingWriter, ShapeWriter
 from openpyxl.writer.charts import ChartWriter
 from openpyxl.writer.worksheet import write_worksheet, write_worksheet_rels
+from openpyxl.writer.comments import CommentWriter
 
 
 class ExcelWriter(object):
@@ -62,13 +74,12 @@ class ExcelWriter(object):
         """Write the various xml files into the zip archive."""
         # cleanup all worksheets
         shared_string_table = self._write_string_table(archive)
-        
+
         archive.writestr(ARC_CONTENT_TYPES, write_content_types(self.workbook))
         archive.writestr(ARC_ROOT_RELS, write_root_rels(self.workbook))
         archive.writestr(ARC_WORKBOOK_RELS, write_workbook_rels(self.workbook))
         archive.writestr(ARC_APP, write_properties_app(self.workbook))
-        archive.writestr(ARC_CORE,
-                write_properties_core(self.workbook.properties))
+        archive.writestr(ARC_CORE, write_properties_core(self.workbook.properties))
         if self.workbook.loaded_theme:
             archive.writestr(ARC_THEME, self.workbook.loaded_theme)
         else:
@@ -76,6 +87,14 @@ class ExcelWriter(object):
         archive.writestr(ARC_STYLE, self.style_writer.write_table())
         archive.writestr(ARC_WORKBOOK, write_workbook(self.workbook))
 
+        if self.workbook.vba_archive:
+            vba_archive = self.workbook.vba_archive
+            for name in vba_archive.namelist():
+                for s in ARC_VBA:
+                    if name.startswith(s):
+                        archive.writestr(name, vba_archive.read(name))
+                        break
+
         self._write_worksheets(archive, shared_string_table, self.style_writer)
 
     def _write_string_table(self, archive):
@@ -88,26 +107,35 @@ class ExcelWriter(object):
 
         return shared_string_table
 
+    def _write_images(self, images, archive, image_id):
+        for img in images:
+            buf = BytesIO()
+            img.image.save(buf, format= 'PNG')
+            archive.writestr(PACKAGE_IMAGES + '/image%d.png' % image_id, buf.getvalue())
+            image_id += 1
+        return image_id
+
     def _write_worksheets(self, archive, shared_string_table, style_writer):
         drawing_id = 1
         chart_id = 1
         image_id = 1
         shape_id = 1
+        comments_id = 1
 
         for i, sheet in enumerate(self.workbook.worksheets):
             archive.writestr(PACKAGE_WORKSHEETS + '/sheet%d.xml' % (i + 1),
                     write_worksheet(sheet, shared_string_table,
                             style_writer.get_style_by_hash()))
-            if sheet._charts or sheet._images or sheet.relationships:
+            if sheet._charts or sheet._images or sheet.relationships or sheet._comment_count > 0:
                 archive.writestr(PACKAGE_WORKSHEETS +
                         '/_rels/sheet%d.xml.rels' % (i + 1),
-                        write_worksheet_rels(sheet, drawing_id))
+                        write_worksheet_rels(sheet, drawing_id, comments_id))
             if sheet._charts or sheet._images:
                 dw = DrawingWriter(sheet)
                 archive.writestr(PACKAGE_DRAWINGS + '/drawing%d.xml' % drawing_id,
                     dw.write())
                 archive.writestr(PACKAGE_DRAWINGS + '/_rels/drawing%d.xml.rels' % drawing_id,
-                    dw.write_rels(chart_id, image_id))
+                    dw.write_rels(chart_id, image_id)) # TODO remove this dependency
                 drawing_id += 1
 
                 for chart in sheet._charts:
@@ -117,20 +145,24 @@ class ExcelWriter(object):
 
                     if chart._shapes:
                         archive.writestr(PACKAGE_CHARTS + '/_rels/chart%d.xml.rels' % chart_id,
-                            cw.write_rels(drawing_id))
+                            cw.write_rels(drawing_id)) # TODO remove this dependency
                         sw = ShapeWriter(chart._shapes)
                         archive.writestr(PACKAGE_DRAWINGS + '/drawing%d.xml' % drawing_id,
-                            sw.write(shape_id))
+                            sw.write(shape_id)) # TODO remove this dependency
                         shape_id += len(chart._shapes)
                         drawing_id += 1
 
                     chart_id += 1
 
-                for img in sheet._images:
-                    buf = StringIO()
-                    img.image.save(buf, format= 'PNG')
-                    archive.writestr(PACKAGE_IMAGES + '/image%d.png' % image_id, buf.getvalue())
-                    image_id += 1
+                image_id = self._write_images(sheet._images, archive, image_id)
+
+            if sheet._comment_count > 0:
+                cw = CommentWriter(sheet)
+                archive.writestr(PACKAGE_XL + '/comments%d.xml' % comments_id,
+                    cw.write_comments())
+                archive.writestr(PACKAGE_XL + '/drawings/commentsDrawing%d.vml' % comments_id,
+                    cw.write_comments_vml())
+                comments_id += 1
 
     def save(self, filename):
         """Write data into the archive."""
